import { TemplateModule } from "../types";
/**
 * Compiles LogiMat to a math function (or multiple). Each function/variable will be on a separate line.
 * @param input {string} - is the input LogiMat code that will be compiled.
 * @param useTex {boolean} - is a value indicating if the output should be converted to Tex.
 * @param noFS {boolean} - is a valid indicating if untrusted filesystem operations should be blocked (for example, code telling the compiler to load an NPM module). This is not a security feature.
 * @param filePath {string} - is a path to the file currently being compiled.
 * @param piecewise {boolean} - is a value indicating if the output should use piecewise instead of pure math for logic.
 * @param strict {boolean} - is a value indicating if an error should be thrown if an undefined function/variable is used.
 * @param outputMaps {boolean} - is a value indicating if the output should be a map of simplified to unsimplified output.
 * @param simplificationMap {Record<string, string>} - is a map of simplified values to unsimplified values.
 * @param importMap {Record<string, TemplateModule>} - is a map of imports which will be used instead of require if specified.
 */
export declare const Compile: (input: string, useTex?: boolean, noFS?: boolean, filePath?: string, piecewise?: boolean, strict?: boolean, outputMaps?: boolean, simplificationMap?: Record<string, string>, importMap?: Record<string, TemplateModule | string>) => Promise<string | {
    output: string[];
    simplificationMap: Record<string, string>;
    importMap: Record<string, TemplateModule | string>;
}>;
