import ohm from "ohm-js";
import { TemplateBlock, TemplateContext } from "../types";
export declare const grammar: ohm.Grammar;
export declare const semantic: ohm.Semantics;
export interface ParserOutput {
    imports: Import[];
    declarations: OuterDeclaration[];
}
export interface Import {
    importType: string;
    path: string;
}
export declare type OuterDeclaration = Template | OuterConstDeclaration | OuterFunctionDeclaration | ExportDeclaration | ActionDeclaration | ActionsDeclaration | ExpressionDeclaration | GraphDeclaration | PointDeclaration | PolygonDeclaration | DisplayDeclaration;
export declare type InternalTemplateArg = string | number | boolean | TemplateBlock | TemplateDeclareValue | TemplateExpression;
export interface TemplateDeclareValue {
    var: true;
    value: string;
}
export interface TemplateExpression {
    expression: true;
    value: Expression;
    source: string;
    nonStrict?: boolean;
}
export interface Template {
    type: "template" | "templatefunction";
    modifier: undefined;
    name: string;
    args: InternalTemplateArg[];
    context: TemplateContext;
}
export interface OuterConstDeclaration {
    type: "const";
    modifier: Modifier;
    name: string;
    expr: Expression;
}
export interface OuterFunctionDeclaration {
    type: "function";
    modifier: Modifier;
    name: string;
    args: string[];
    block: Statement[];
}
export interface ExportDeclaration {
    type: "export";
    modifier: "export";
    name: string;
}
export interface ActionDeclaration {
    type: "action";
    modifier: Modifier;
    name: string;
    funcName: string;
    args: string[];
    block: Statement[];
}
export interface ActionsDeclaration {
    type: "actions";
    modifier: Modifier;
    name: string;
    args: string[][];
    actionArgs: string[];
}
export interface ExpressionDeclaration {
    type: "expression";
    modifier: Modifier;
    block: Statement[];
}
export interface GraphDeclaration {
    type: "graph";
    modifier: Modifier;
    p1: Expression;
    p2: Expression;
    op: string;
}
export interface PointDeclaration {
    type: "point";
    modifier: Modifier;
    point: Expression;
}
export interface PolygonDeclaration {
    type: "polygon";
    modifier: Modifier;
    points: Expression[];
}
export interface DisplayDeclaration {
    type: "display";
    modifier: Modifier;
    displayType: "color" | "stroke" | "thickness" | "fill" | "click" | "label" | "drag" | "hidden" | "outline" | "angle" | "size" | "min" | "max" | "step";
    value: string | Expression | TemplateString | ParsedActionArgs;
}
export interface TemplateString {
    type: "tstring";
    args: (string | Expression)[];
}
export interface ParsedActionArgs {
    type: "aargs";
    name: string;
    args: (string | Expression)[];
}
export interface Expression {
    type: "f" | "^" | "*" | "/" | "+" | "-" | "n" | "a_m" | "a_f" | "b" | "v" | "sum" | "prod" | "int" | "div";
    args: (string | object)[];
}
export declare type Statement = ConstDeclaration | Template | SetState | IfStatement;
export interface ConstDeclaration {
    type: "const";
    name: string;
    expr: Expression;
}
export interface SetState {
    type: "state";
    expr: Expression;
}
export interface IfStatement {
    type: "if";
    condition: Expression;
    ifaction: Statement[];
    elseaction: Statement[];
}
export declare type Modifier = "export" | "inline";
